'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var hoistNonReactStatics = _interopDefault(require('hoist-non-react-statics'));
var invariant = _interopDefault(require('invariant'));
var PropTypes = _interopDefault(require('prop-types'));
var ramdaExtension = require('ramda-extension');
var React = require('react');
var React__default = _interopDefault(React);
var namespacesReact = require('@redux-tools/namespaces-react');
var utilsReact = require('@redux-tools/utils-react');
var ramda = require('ramda');
var reactRedux = require('react-redux');
var injectors = require('@redux-tools/injectors');
var namespaces = require('@redux-tools/namespaces');

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

var makeDecorator = function makeDecorator(storeInterface, useInjectables) {
  invariant(ramdaExtension.isObject(storeInterface), 'The store interface is undefined.');
  var type = storeInterface.type;
  var decoratorName = type ? "With" + ramdaExtension.toPascalCase(type) : 'Injector';
  return function (injectables, options) {
    if (options === void 0) {
      options = {};
    }

    return function (NextComponent) {
      var Injector = function Injector(props) {
        var _ref, _options$feature, _ref2, _ref3, _options$namespace;

        // eslint-disable-next-line react/destructuring-assignment
        var feature = (_ref = (_options$feature = options.feature) != null ? _options$feature : props.feature) != null ? _ref : null;
        var contextNamespace = namespacesReact.useNamespace(feature); // eslint-disable-next-line react/destructuring-assignment

        var namespace = (_ref2 = (_ref3 = (_options$namespace = options.namespace) != null ? _options$namespace : props.namespace) != null ? _ref3 : contextNamespace) != null ? _ref2 : null;
        var isInitialized = useInjectables(injectables, _extends({}, options, {
          feature: feature,
          namespace: namespace
        }));

        if (isInitialized) {
          return React__default.createElement(NextComponent, _extends({
            key: String([feature, namespace])
          }, props));
        }

        return null;
      };

      Injector.propTypes = {
        feature: PropTypes.string,
        namespace: PropTypes.string
      };
      hoistNonReactStatics(Injector, NextComponent);
      Injector.displayName = decoratorName + "(" + utilsReact.getDisplayName(NextComponent) + ")";
      return Injector;
    };
  };
};

var IS_SERVER = Boolean(typeof process !== 'undefined' && process && process.versions && process.versions.node);

var useUniversalLayoutEffect = IS_SERVER ? React.useEffect : React.useLayoutEffect;
var getOtherProps = ramda.omit(['isGlobal', 'global', 'isPersistent', 'persist']);

var makeHook = function makeHook(storeInterface) {
  invariant(ramdaExtension.isObject(storeInterface), 'The store interface is undefined.');
  var getEntries = storeInterface.getEntries,
      ejectionKey = storeInterface.ejectionKey,
      injectionKey = storeInterface.injectionKey,
      type = storeInterface.type;
  var pascalCaseType = ramdaExtension.toPascalCase(type);
  var hookName = "use" + pascalCaseType;

  var useInjectables = function useInjectables(injectables, options) {
    var _ref, _options$isGlobal, _ref2, _options$isPersistent, _options$isNamespaced, _options$feature, _options$namespace;

    if (options === void 0) {
      options = {};
    }

    var locationMessages = ["@redux-tools " + type, injectables];

    var warn = function warn() {
      var _console;

      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      return (_console = console).warn.apply(_console, locationMessages.concat(args));
    }; // NOTE: `options.global` and `options.persist` are deprecated.


    var isGlobal = (_ref = (_options$isGlobal = options.isGlobal) != null ? _options$isGlobal : options.global) != null ? _ref : false;
    var isPersistent = (_ref2 = (_options$isPersistent = options.isPersistent) != null ? _options$isPersistent : options.persist) != null ? _ref2 : false;
    var isNamespaced = (_options$isNamespaced = options.isNamespaced) != null ? _options$isNamespaced : false;
    var feature = (_options$feature = options.feature) != null ? _options$feature : null;
    var contextNamespace = namespacesReact.useNamespace(feature);

    var _useContext = React.useContext(reactRedux.ReactReduxContext),
        store = _useContext.store;

    var _useContext2 = React.useContext(namespacesReact.NamespaceContext),
        isUseNamespaceProvided = _useContext2.isUseNamespaceProvided;

    var namespace = isGlobal ? null : (_options$namespace = options.namespace) != null ? _options$namespace : contextNamespace;
    var inject = store[injectionKey];
    var eject = store[ejectionKey]; // NOTE: On the server, the injectables should be injected beforehand.

    var _useState = React.useState(IS_SERVER),
        isInitialized = _useState[0],
        setIsInitialized = _useState[1];

    var props = ramdaExtension.rejectNil(_extends({}, getOtherProps(options), {
      feature: feature,
      namespace: namespace
    })); // TODO: Refactor when React DevTools support multiple debug values or non-primitive structures.

    React.useDebugValue(String(["Namespace: " + namespace, "Feature: " + feature, "Type: " + pascalCaseType, "Initialized: " + isInitialized]));

    if (IS_SERVER) {
      var areEntriesAlreadyInjected = ramda.all(function (entry) {
        return ramda.includes(entry, getEntries(store));
      }, injectors.createEntries(injectables, props));

      if (!areEntriesAlreadyInjected) {
        warn("When rendering on the server, inject all " + type + " before calling", "'ReactDOMServer.renderToString()'. You should do this inside an", "'async getInitialProps()' function, i.e. where you fetch data and", 'do other side effects. If you need to do server-side injections', 'during rendering, open an issue.');
      }
    }

    var effectDependencies = [namespace, feature, isGlobal, isPersistent, inject, eject, injectables]; // NOTE: This doesn't run on the server, but won't trigger `useLayoutEffect` warnings either.

    useUniversalLayoutEffect(function () {
      if (isGlobal && ramdaExtension.isNotNil(feature) && feature !== namespaces.DEFAULT_FEATURE) {
        warn("You are using a feature (" + feature + ") with global " + type + ".", 'This will have no effect.');
      }

      if (isUseNamespaceProvided && ramda.isNil(namespace) && !isGlobal) {
        warn("You're injecting " + type + ", but the namespace could not be resolved from React context!", 'They will be injected globally. If this is intended, consider passing', "'isGlobal: true' to the injector, e.g. '" + hookName + "(" + type + ", { isGlobal: true })'.");
      }

      if (ramdaExtension.isNotNil(options.global)) {
        warn("'global: " + options.global + "' is deprecated. Use 'isGlobal: " + options.global + "'.");
      }

      if (ramdaExtension.isNotNil(options.persist)) {
        warn("'persist: " + options.persist + "' is deprecated. Use 'isPersistent: " + options.persist + "'.");
      }

      invariant(!isNamespaced || ramdaExtension.isNotNil(namespace), "You're injecting " + type + " marked as namespaced, but no namespace could be resolved.");
      invariant(inject, "'store." + injectionKey + "' missing. Are you using the enhancer correctly?");
      invariant(eject, "'store." + ejectionKey + "' missing. Are you using the enhancer correctly?");
      inject(injectables, props);
      setIsInitialized(true);
      return function () {
        if (!isPersistent) {
          eject(injectables, props);
        }
      };
    }, effectDependencies);
    return isInitialized;
  };

  return useInjectables;
};

exports.makeDecorator = makeDecorator;
exports.makeHook = makeHook;
